//############################################################################
//
// LaserBoy !!!
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#ifndef __BMP_FUNCTIONS__
#define __BMP_FUNCTIONS__

#include "ez_uail_common.hpp"
#include "ez_font.hpp"

//############################################################################
extern "C"
{
//############################################################################
struct ezfb_bitmap
{                               //  defaults     offset
    char      B               ; //  'B'          0 :
    char      M               ; //  'M'          1 :
    u_int     file_size       ; //               2 :
    int       reserved        ; //  0            6 :
    int       jump            ; //              10 : offset to pixel data
    int       to_end          ; //  40          14 : from here to end of header
    u_int     xres            ; //              18 :
    u_int     yres            ; //              22 :
    u_short   planes          ; //  1           26 :
    u_short   bpp             ; //              28 : bits per pixel
    int       comp            ; //  0           30 : no compression
    u_int     image_size      ; //              34 : in bytes
    u_int     xppm            ; //  2835        38 : pixels per meter
    u_int     yppm            ; //  2835        42 : approx. 72 dots per inch
    u_int     c_used          ; //              46 :
    u_int     c_important     ; //              50 : ................
    u_int     black           ; //              54 : non standard   |
    u_int     white           ; //              58 : added for ezfb |
    u_int     gray            ; //              58 :                |
    int       fd              ; //              62 :                |
    u_int     bytes_per_line  ; //              66 :                |
    u_int     palette_size    ; //              70 :                |
    u_char*   data            ; //  null        74 :                |
    u_int*    palette         ; //  null        78 :                |
    u_char*   image           ; //  null        82 :                |
    u_char**  uchar_row       ; //  null        86 :                |
    u_short** ushort_row      ; //  null        90 :                |
    u_int**   uint_row        ; //  null        94 :                |
    // function pointers
    u_int  (*make_rgb     )(u_char r, u_char g, u_char b);
    u_char (*r_from_rgb   )(u_int rgb                   );
    u_char (*g_from_rgb   )(u_int rgb                   );
    u_char (*b_from_rgb   )(u_int rgb                   );
    int    (*put_pixel    )(struct ezfb_bitmap* bmp, u_int x, u_int y, u_int c);
    int    (*get_pixel    )(struct ezfb_bitmap* bmp, u_int x, u_int y);
    int    (*get_pixel_rgb)(struct ezfb_bitmap* bmp, u_int x, u_int y);
};

//############################################################################
struct ezfb_rectangle
{
    int x1    ;
    int y1    ;
    int x2    ;
    int y2    ;
    int height;
    int width ;
};

//############################################################################
//  UTILITY
void  ezfb_dump_rectangle (struct ezfb_rectangle* area, char* message);
void  bmp_dump_header     (struct ezfb_bitmap* bmp, char* file_name, char* message);

u_int ezfb_rgb_hue        (struct ezfb_bitmap* bmp, u_short hue);
u_int ezfb_rgb_tint       (struct ezfb_bitmap* bmp, u_short hue, u_char tint );
u_int ezfb_rgb_shade      (struct ezfb_bitmap* bmp, u_short hue, u_char shade);

//############################################################################
inline int bmp_set_palette_index(struct ezfb_bitmap* bmp, u_int index, u_char r, u_char g, u_char b)
{
    if(     bmp->bpp <= 8
        && (index < (u_int)(0x01 << bmp->bpp))
      )
    {
        bmp->palette[index] = ((r << 16) | (g << 8) | (b));
        return 1;
    }
    else
        return 0;
}

//############################################################################
inline u_int bmp_get_palette_index(struct ezfb_bitmap* bmp, u_int index)
{
    if(     bmp->bpp <= 8
        && (index < (u_int)(0x01 << bmp->bpp))
      )
        return bmp->palette[index];
    return 0;
}

//############################################################################
inline u_char bmp_get_palette_index_r(struct ezfb_bitmap* bmp, u_int index)
{
    return (bmp_get_palette_index(bmp, index) & 0x00ff0000) >> 16;
}

//############################################################################
inline u_char bmp_get_palette_index_g(struct ezfb_bitmap* bmp, u_int index)
{
    return (bmp_get_palette_index(bmp, index) & 0x0000ff00) >> 8;
}

//############################################################################
inline u_char bmp_get_palette_index_b(struct ezfb_bitmap* bmp, u_int index)
{
    return bmp_get_palette_index(bmp, index) & 0x000000ff;
}

//############################################################################
inline int bmp_is_same_size(struct ezfb_bitmap* a, struct ezfb_bitmap* b)
{
    if(    a->bpp  == b->bpp
        && a->xres == b->xres
        && a->yres == b->yres
      )
        return 1;
    return 0;
}

//############################################################################
//  BITMAP IN MEMORY
void        bmp_init              (struct ezfb_bitmap* bmp, u_int x, u_int y, u_short bpp);
long double bmp_put_line          (struct ezfb_bitmap* bmp, int x1, int y1, int x2, int y2, u_int c);
void        bmp_put_line_fast     (struct ezfb_bitmap* bmp, int x1, int y1, int x2, int y2, u_int c);
long double bmp_put_ray           (struct ezfb_bitmap* bmp, u_int x, u_int y, long double r, long double t, u_int c);
void        bmp_put_ray_fast      (struct ezfb_bitmap* bmp, u_int x, u_int y, long double r, long double t, u_int c);
void        bmp_put_rectangle     (struct ezfb_bitmap* bmp, int x1, int y1, int x2, int y2, u_int c, int filled);
int         bmp_put_byte_pattern  (struct ezfb_bitmap* bmp, u_char  byte,  int x, int y, int fc, int bc, int dir, int size);
int         bmp_put_ascii         (struct ezfb_bitmap* bmp, u_char  ascii, int x, int y, int fc, int bc, int dir, int size);
int         bmp_put_string        (struct ezfb_bitmap* bmp, char*   str,   int x, int y, int fc, int bc, int dir, int size);
int         bmp_printf            (struct ezfb_bitmap* bmp,                int x, int y, int fc, int bc, int dir, int size, const char* text_format, ...);
void        bmp_dump_palette      (struct ezfb_bitmap* bmp);
void        bmp_dump_image        (struct ezfb_bitmap* bmp);
void        bmp_clear             (struct ezfb_bitmap* bmp, u_char c);
void        bmp_uint              (struct ezfb_bitmap* bmp, u_int c);
void        bmp_copy              (struct ezfb_bitmap* target, struct ezfb_bitmap* original);
void        bmp_flip              (struct ezfb_bitmap* bmp);
void        bmp_into_bmp          (struct ezfb_bitmap* target, struct ezfb_bitmap* insert);
void        bmp_free              (struct ezfb_bitmap* bmp);
//----------------------------------------------------------------------------
//  BITMAP MEMORY AND FILE
int         bmp_from_file         (struct ezfb_bitmap* bmp, char* file_name);
int         bmp_save_as           (struct ezfb_bitmap* bmp, char* file_name);
//----------------------------------------------------------------------------
//  BITMAP MEMORY AND SDL
void        bmp_clone_SDL_Surface (struct ezfb_bitmap* bmp, struct SDL_Surface* screen, int copy);
void        bmp_to_SDL_Surface    (struct ezfb_bitmap* bmp, struct SDL_Surface* screen);

//############################################################################
} // end extern "C"

#endif // __BMP_FUNCTIONS__

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
