//############################################################################
//
// LaserBoy !!!
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#include "ez_uail_gui.hpp"
    
//############################################################################
ez_palette& ez_palette::reorder()
{
    ez_palette reordered;
    reordered.reserve(size());
    sort(begin(), end()); // sort this
    for(int i = 0; i < number_of_colors(); i++)
        if(at(i) != reordered.back()) // remove duplicates!
            reordered.push_back(at(i));
    *this = reordered;
    return *this;
}

//############################################################################
int ez_palette::from_ifstream_uail(ifstream& in, const ez_uail_header& header)
{
    int       i,
              error = EZ_UAIL_OK;
    ez_color  color;
    clear();
    //------------------------------------------------------------------------
    reserve(header.quantity);
    //------------------------------------------------------------------------
    for(i = 0; i < header.quantity; i++)
    {
        if((error = color.from_ifstream_uail(in)))
            break;
        push_back(color);
    }
    find_factors();
    name = header.name;
    return error;
}

//############################################################################
void ez_palette::to_ofstream_uail(ofstream& out, ez_uail_header& header) const
{
    int i;
    header.quantity = (u_short)number_of_colors();
    header.to_ofstream_uail(out);
    for(i = 0; i < number_of_colors(); i++)
        at(i).to_ofstream_uail(out);
    return;
}

//############################################################################
int ez_palette::from_ascii_file(string file)
{
    int          color_count = 0,
                 color_value,
                 error = EZ_UAIL_OK;
    ez_color     color;
    string       junk;

    if(    file.size() < 4
        || file.substr(file.size() - 4) != ".pal"
      )
        file += ".pal";

    ifstream     in((EZ_UAIL_PALETTE_SHARE + file).c_str(), ios::in);

    if(in.is_open())
    {
        in >> name;
        in >> junk; // Red
        in >> junk; // Green
        in >> junk; // Blue
        reserve(EZ_PALETTE_MAX);
        while(!in.eof() && (color_count < EZ_PALETTE_MAX))
        {
            in >> color_value;    color.r = (char)color_value;
            in >> color_value;    color.g = (char)color_value;
            in >> color_value;    color.b = (char)color_value;
            push_back(color);
            color_count++;
        }
        in.close();
        find_factors();
    }
    else
        error = EZ_UAIL_FILE_OPEN_FAILED;
    return error;
}

//############################################################################
void ez_palette::save_as_ascii(string file) const
{
    if(    file.size() < 4
        || file.substr(file.size() - 4) != ".pal"
      )
        file += ".pal";
    ofstream out((EZ_UAIL_PALETTE_SHARE + file).c_str(), ios::out);
    if(out.is_open())
    {
        out << name
            << endl
            << "Red     Green   Blue"
            << endl;
        for(int i = 0; i < number_of_colors(); i++)
            out << setw(3)
                << (int)at(i).r
                << setw(4)
                << (int)at(i).g
                << setw(4)
                << (int)at(i).b
                << endl;
        out << endl;
        out.close();
    }
    return;
}

//############################################################################
int ez_palette::from_ezfb_bmp_palette()
{
    int i;
    if(BMP.bpp == 8)
    {
        clear();
        reserve(EZ_PALETTE_MAX);
        for(i = 0; i < EZ_PALETTE_MAX; i++)
            set_palette_item( i,
                              bmp_get_palette_index_r(&BMP, i),
                              bmp_get_palette_index_g(&BMP, i),
                              bmp_get_palette_index_b(&BMP, i)
                            );
        first = 0;
        last  = 255;
        black = BMP.black ;
        white = BMP.white ;
        return EZ_UAIL_OK;
    }
    return EZ_UAIL_INCOMPATIBLE_FORMATS;
}

//############################################################################
int ez_palette::to_ezfb_bmp_palette() const
{
    return to_ezfb_bmp_palette(&BMP);
}

//############################################################################
int ez_palette::to_ezfb_bmp_palette(struct ezfb_bitmap* bmp) const
{
    int i;
    if(bmp->bpp == 8)
    {
        for(i = 0; i < EZ_PALETTE_MAX; i++)
            if(i < number_of_colors())
                bmp_set_palette_index(bmp, i, at(i).r, at(i).g, at(i).b);
            else
                bmp_set_palette_index(bmp, i, 0, 0, 0);
        bmp->black = black ;
        bmp->white = white ;
        return EZ_UAIL_OK;
    }
    return EZ_UAIL_INCOMPATIBLE_FORMATS;
}

//############################################################################
void ez_palette::find_factors()
{
    int    i       ,
           j       ,
           k = 255 ;
    //------------------------------------------------------------------------
    first = black = white = last = 0;
    //------------------------------------------------------------------------
    for(i = 0; i < number_of_colors(); i++)
    {
        if(at(black) > at(i))
          black = i;
    //------------------------------------------------------------------------
        if(at(white) < at(i))
          white = i;
    //------------------------------------------------------------------------
        j =   abs(at(i).r - 0x80)
            + abs(at(i).g - 0x80)
            + abs(at(i).b - 0x80);
        if(j < k)
            k    = j;
    }
    //------------------------------------------------------------------------
    for(i = 0; i < number_of_colors(); i++)
        if(at(i) > 0)
        {
            first = i;
            break;
        }
    //------------------------------------------------------------------------
    for(i = number_of_colors() - 1; i >= 0 ; i--)
        if(at(i) > 0)
        {
            last = i;
            break;
        }
    //------------------------------------------------------------------------
    return;
}

//############################################################################
int ez_palette::best_match(ez_color rgb) const
{
    int    i          ,
           j          ,
           k     = 255,
           match = 0  ;
    //------------------------------------------------------------------------
    for(i = 0; i < number_of_colors(); i++)
    {
        j =   abs(at(i).r - rgb.r)
            + abs(at(i).g - rgb.g)
            + abs(at(i).b - rgb.b);
        if(j < k)
        {
            match = i;
            k     = j;
        } 
    }
    //------------------------------------------------------------------------

    if(    AIL->no_black_match
        && (match == black)
      )
        match = white;
    //------------------------------------------------------------------------
    return match;
}

//############################################################################
void ez_palette::best_reduction() // reduces a color_table to a palette
{
    int    i          ,
           j          ,
           k     = 255,
           match = 0  ;
    //------------------------------------------------------------------------
    reorder();
    //------------------------------------------------------------------------
    if(number_of_colors() >= EZ_PALETTE_MAX)
    {
        for(k = 1; k < 765; k++) // minimum difference
        {
            for(i = 0; i < number_of_colors() - 1; i++)
            {
                j =   abs(at(i).r - at(i + 1).r)
                    + abs(at(i).g - at(i + 1).g)
                    + abs(at(i).b - at(i + 1).b);                    
                if(j <= k)
                    erase(begin() + i + 1);
            }
            if(number_of_colors() <= EZ_PALETTE_MAX)
                break;
        }
    }
    //------------------------------------------------------------------------
    find_factors();
    return;
}

//############################################################################
void ez_palette::shade(u_char shade) // 0 shade is no change 255 is black
{
    if(shade)
    {
        for(int i = 0; i < number_of_colors(); i++)
        {
            (at(i).r - shade > 0) ? (at(i).r -= shade) : (at(i).r = 0);
            (at(i).g - shade > 0) ? (at(i).g -= shade) : (at(i).g = 0);
            (at(i).b - shade > 0) ? (at(i).b -= shade) : (at(i).b = 0);
        }
    }
    return;
}

//############################################################################
void ez_palette::tint(u_char tint) // 0 tint is no change 255 is white
{
    if(tint)
    {
        for(int i = 0; i < number_of_colors(); i++)
        {
            (at(i).r + tint < 255) ? (at(i).r += tint) : (at(i).r = 255);
            (at(i).g + tint < 255) ? (at(i).g += tint) : (at(i).g = 255);
            (at(i).b + tint < 255) ? (at(i).b += tint) : (at(i).b = 255);
        }
    }
    return;
}

//############################################################################
bool ez_palette::unite(const ez_palette& palette)
{
    if(    palette.number_of_colors() >  0
        && palette.number_of_colors() <= EZ_PALETTE_MAX
      )
    {
        int            i,
                       j;
        ez_palette     super_palette(*this);
        //--------------------------------------------------------------------
        super_palette.reserve(EZ_PALETTE_MAX);
        //--------------------------------------------------------------------
        for(i = 0; i < palette.number_of_colors(); i++)
        {
            for(j = 0; j < super_palette.number_of_colors(); j++)
                if(super_palette[j] == palette.at(i))
                    break;
            if(j == super_palette.number_of_colors())
                super_palette.push_back(palette.at(i));
        }
        //--------------------------------------------------------------------
        if(super_palette.number_of_colors() <= EZ_PALETTE_MAX)
        {
            super_palette.reorder();
            super_palette.find_factors();
            *this = super_palette;
            return true;
        }
        //--------------------------------------------------------------------
        else
            return false;
    }
    //------------------------------------------------------------------------
    return false;
}

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
